import numpy as np
import pandas as pd


def generate_user_properties():
    np.random.seed(42)  # For reproducible results

    user_ids = np.arange(1, 101)
    ages = np.random.randint(18, 71, size=100)
    genders = np.random.choice(['Male', 'Female'], size=100, p=[0.5, 0.5])
    countries = ['USA', 'Canada', 'UK', 'Germany', 'France', 'Australia', 'Japan', 'Brazil', 'India', 'China']
    country_distribution = np.random.choice(countries, size=100)
    device_types = np.random.choice(['Mobile', 'Desktop', 'Tablet'], size=100, p=[0.5, 0.3, 0.2])
    membership_status = np.random.choice(['Free', 'Premium'], size=100, p=[0.7, 0.3])
    user_properties = pd.DataFrame({
        'userid': user_ids,
        'age': ages,
        'gender': genders,
        'country': country_distribution,
        'device_type': device_types,
        'membership_status': membership_status
    })
    return user_properties


def generate_event_data():
    np.random.seed(42)  # For reproducible results

    # Expanding the event_list DataFrame
    user_ids = np.random.randint(1, 9, size=100)
    event_names = np.random.choice(['login', 'purchase'], size=100, p=[0.7, 0.3])
    purchase_values = np.where(event_names == 'purchase', np.random.randint(50, 350, size=100), 0)
    timestamps = pd.date_range(start="2023-01-01", periods=100, freq='H')

    event_list = pd.DataFrame({
        'timestamp': timestamps,
        'userid': user_ids,
        'event_name': event_names,
        'purchase_value': purchase_values,
    })
    event_list = event_list[["timestamp", "userid", "event_name", "purchase_value"]]
    return event_list


def generate_user_allocations():
    np.random.seed(42)  # For reproducible results
    user_ids = np.arange(1, 101)
    abgroups = np.random.choice(['A', 'B'], size=100, p=[0.5, 0.5])
    ab_timestamps = pd.date_range(start="2022-12-15", periods=100, freq='H')

    ab_test_allocations = pd.DataFrame({
        'timestamp': ab_timestamps,
        'userid': user_ids,
        'abgroup': abgroups,
    })
    ab_test_allocations = ab_test_allocations[["timestamp", "userid", "abgroup"]]
    return ab_test_allocations



import unittest
import pandas as pd

from ab_test_advanced_toolkit.metrics import Metric
from ab_test_advanced_toolkit.stat_significance import StatSignificanceResult
from data_generation.data_generator import generate_synthetic_data, run_analysis

class TestDataGenerator(unittest.TestCase):
    
    def test_generate_synthetic_data(self):
        num_users = 1000
        countries = ['US', 'UK', 'DE', 'FR', 'CA', 'AU', 'JP', 'IN']
        platforms = ['iOS', 'Android', 'Web', 'Desktop']
        user_segments = ['Segment_1', 'Segment_2', 'Segment_3', 'Segment_4']
        ab_groups = ['a1', 'a2', 'b']
        
        data = generate_synthetic_data(
            num_users=num_users,
            countries=countries,
            platforms=platforms,
            user_segments=user_segments,
            ab_groups=ab_groups,
            noise_level=1.0,
            base_increase_percentage=0.05,
            correlation_level=0.5,
        )
        
        # Check if the data is generated correctly
        self.assertEqual(len(data), num_users)
        self.assertTrue('userid' in data.columns)
        self.assertTrue('country' in data.columns)
        self.assertTrue('platform' in data.columns)
        self.assertTrue('user_segment' in data.columns)
        self.assertTrue('abgroup' in data.columns)
        self.assertTrue('value' in data.columns)
        self.assertTrue('pre_test_value' in data.columns)

    def test_run_analysis(self):
        num_users = 1000
        countries = ['US', 'UK', 'DE', 'FR', 'CA', 'AU', 'JP', 'IN']
        platforms = ['iOS', 'Android', 'Web', 'Desktop']
        user_segments = ['Segment_1', 'Segment_2', 'Segment_3', 'Segment_4']
        ab_groups = ['a1', 'a2', 'b']
        
        data = generate_synthetic_data(
            num_users=num_users,
            countries=countries,
            platforms=platforms,
            user_segments=user_segments,
            ab_groups=ab_groups,
            noise_level=1.0,
            base_increase_percentage=0.05,
            correlation_level=0.5,
        )
        
        results = run_analysis(data)
        
        # Check if the analysis results are not empty
        self.assertTrue("no_enhancement" in results)
        self.assertTrue("cuped" in results)
        self.assertTrue("gboost_cuped" in results)

        # output type is Metric
        self.assertIsInstance(results["no_enhancement"], Metric)
        self.assertIsInstance(results["cuped"], Metric)
        self.assertIsInstance(results["gboost_cuped"], Metric)


if __name__ == '__main__':
    unittest.main()
